import { EventsHandler, IEventHandler } from '@nestjs/cqrs';
import { STATUS } from 'src/core/strings/constants/base.constants';
import { PaymentMethodDataService } from 'src/modules/transaction/payment-method/data/services/payment-method-data.service';
import { TransactionDataService } from 'src/modules/transaction/transaction/data/services/transaction-data.service';
import { TransactionChangeStatusEvent } from 'src/modules/transaction/transaction/domain/entities/event/transaction-change-status.event';
import { sendEmail } from '../helpers/send-email.helper';
import { TransactionPaymentType } from 'src/modules/transaction/transaction/constants';

@EventsHandler(TransactionChangeStatusEvent)
export class PaymentTransactionHandler
  implements IEventHandler<TransactionChangeStatusEvent>
{
  constructor(
    private dataService: TransactionDataService,
    private paymentService: PaymentMethodDataService,
  ) {}

  async handle(event: TransactionChangeStatusEvent) {
    const data_id = event.data.id;
    const old_data = event.data.old;
    const current_data = event.data.data;
    let payments = [];

    if (
      old_data.status == STATUS.DRAFT &&
      current_data.status == STATUS.PENDING &&
      current_data.payment_type != TransactionPaymentType.COUNTER &&
      !!current_data.customer_email
    ) {
      if (current_data.payment_type != TransactionPaymentType.MIDTRANS) {
        payments = await this.paymentService.getManyByOptions({
          where: {
            status: STATUS.ACTIVE,
          },
        });
      }

      const transaction = await this.dataService.getOneByOptions({
        where: {
          id: data_id,
        },
        relations: ['items'],
      });

      try {
        sendEmail(
          [
            {
              ...transaction,
              email: transaction.customer_email,
              payment_methods: payments,
            },
          ],
          'Payment Confirmation',
        );
      } catch (error) {
        console.log(error);
      }
    }
  }
}
