import { STATUS } from 'src/core/strings/constants/base.constants';
import {
  TransactionPaymentType,
  TransactionType,
} from 'src/modules/transaction/transaction/constants';

export function mappingTransaction(data) {
  let payment_type_bank: any = null;
  const season_period = {
    id: data.season_period_id,
    holiday_name: data.season_period_name,
    season_type: {
      id: data.season_period_type_id,
      name: data.season_period_type_name,
    },
  };

  if (data.payment_type_method_id || data.payment_type_method_name) {
    payment_type_bank = {
      id: data.payment_type_method_id,
      issuer_name: data.payment_type_method_name,
      account_number: data.payment_type_method_number,
      qr_image: data.payment_type_method_qr,
    };
  }

  const items = data?.['items']?.map((itemData) => {
    let tenant;

    if (itemData.item_tenant_id) {
      tenant = {
        id: itemData.item_tenant_id,
        name: itemData.item_tenant_name,
        share_margin: itemData.item_tenant_share_margin,
      };
    }

    return {
      item: {
        id: itemData.item_id,
        name: itemData.item_name,
        item_type: itemData.item_type,
        base_price: itemData.item_price,
        hpp: itemData.item_hpp,
        tenant: tenant,
        item_category: {
          id: itemData.item_category_id,
          name: itemData.item_category_name,
        },
      },
      id: itemData.id,
      refund: itemData.refund,
      qty: itemData.qty,
      qty_remaining: itemData.qty_remaining,
      total_price_refund: itemData.refund?.refund_total ?? 0,
      total_price: itemData.total_price,
    };
  });

  Object.assign(data, {
    season_period: season_period,
    items: items,
    payment_type_bank: payment_type_bank,
    payment_code: data.payment_code_reference,
  });

  delete data.season_period_id;
  delete data.season_period_name;
  delete data.season_period_type_id;
  delete data.season_period_type_name;

  delete data.payment_type_method_id;
  delete data.payment_type_method_name;
  delete data.payment_type_method_number;
  delete data.payment_type_method_qr;
}

export function mappingRevertTransaction(data, type) {
  if (type == TransactionType.COUNTER) {
    if (data.booking_id) {
      Object.assign(data, {
        editor_id: data.pos_admin?.id,
        editor_name: data.pos_admin?.name,
        edited_at: new Date(data.created_at),
      });
    } else {
      Object.assign(data, {
        creator_id: data.pos_admin?.id,
        creator_name: data.pos_admin?.name,
      });
    }

    Object.assign(data, {
      id: data.booking_id ?? data._id,
      invoice_code: data.code,
      creator_counter_no: Number(data.pos_number),
      status: STATUS.SETTLED,
      settlement_date: new Date(data.created_at),
      payment_date: new Date(data.created_at),
      invoice_date: new Date(data.created_at),
      payment_type:
        data.payment_type == 'cc'
          ? TransactionPaymentType.CC
          : data.payment_type,
      payment_card_information: data.card_information,
      payment_code_reference: data.payment_code,
      discount_code_id: data.discount_code?.id,
      discount_code: data.discount_code?.code,
      discount_percentage: data.discount_code?.discount,
    });
  } else {
    // Object.assign(data, {
    //   payment_type:
    // })
  }

  Object.assign(data, {
    type: type,
    payment_total_net_profit: data.payment_total,
    customer_category_id: data.customer_category?.id ?? null,
    customer_category_name: data.customer_category?.name ?? null,
    season_period_id: data.season_period?.id ?? null,
    season_period_name: data.season_period?.holiday_name ?? null,
    season_period_type_id: data.season_period?.season_type?.id ?? null,
    season_period_type_name: data.season_period?.season_type?.name ?? null,
    payment_type_method_id:
      data.payment_type_method?.id ?? data.payment_type_bank?.id,
    payment_type_method_number:
      data.payment_type_method?.account_number ??
      data.payment_type_bank?.account_number,
    payment_type_method_name:
      data.payment_type_method?.issuer_name ??
      data.payment_type_bank?.issuer_name,
    payment_type_method_qr:
      data.payment_type_method?.qr_image ?? data.payment_type_bank?.qr_image,
  });

  data.items?.map((item) => {
    const total_price =
      Number(item.item.price ?? item.item.base_price) * Number(item.qty);
    const share_margin = item.item.tenant?.share_margin ?? 0;
    const total_share_tenant =
      share_margin > 0 ? (Number(share_margin) / 100) * total_price : 0;

    Object.assign(item, {
      item_id: item.item.id,
      item_name: item.item.name,
      item_type: item.item.item_type,
      item_price: item.item.base_price,
      item_hpp: item.item.hpp,
      qty_remaining: item.qty,

      item_category_id: item.item.item_category?.id,
      item_category_name: item.item.item_category?.name,
      item_bundlings: item.item.bundling_items?.map(
        (bundling) => bundling.name,
      ),

      item_tenant_id: item.item.tenant?.id ?? null,
      item_tenant_name: item.item.tenant?.id ?? null,
      item_tenant_share_margin: item.item.tenant?.share_margin ?? null,

      total_price: total_price,
      total_hpp: Number(item.item.item_hpp) * Number(item.qty),
      total_share_tenant: total_share_tenant,
      total_profit: total_price - Number(total_share_tenant),
    });
  });
}
